import unittest

from pyramid.config import Configurator
from pyramid.security import Everyone, Allow
from pyramid.view import view_config
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy

from webtest import TestApp


def stupid_callback(uid, *args, **kw):
    """Simple callback"""
    return []


class RootFactory(object):
    """
        Simple RootFactory used for tests
    """
    __acl__ = [
        (Allow, Everyone, 'view'),
        (Allow, 'permission:administor', 'admin'),
    ]

    def __init__(self, request):
        pass


@view_config(route_name='something', renderer='something.mako', permission='view')
def view_something(request):
    """A view of something"""
    return {}


class ViewTests(unittest.TestCase):

    def _get_app(self, settings):
        config = Configurator(settings=settings, root_factory=RootFactory)
        authorization_policy = ACLAuthorizationPolicy()
        config.set_authorization_policy(authorization_policy)
        authn_policy = AuthTktAuthenticationPolicy(
                    'sosecret', callback=stupid_callback, hashalg='sha512')
        config.set_authentication_policy(authn_policy)
        config.include('pyramid_mako')
        config.add_route('something', '/something')
        config.scan()
        app = TestApp(config.make_wsgi_app())
        return app


    def test_tween_maintenance(self):
        # Tween not yet enabled
        settings = {
            'mako.directories': 'pyramid_maintenance:tests/templates',
        }
        app = self._get_app(settings)
        content = app.get('/something')
        self.assertTrue('This is something' in content)

        # With tween enabled
        settings = {
            'mako.directories': 'pyramid_maintenance:tests/templates',
            'pyramid.tweens': 'pyramid_maintenance.tween_maintenance',
            'pyramid_maintenance.permissions': 'admin',
            'pyramid_maintenance.template': 'maintenance.mako',
        }
        app = self._get_app(settings)
        content = app.get('/something')
        self.assertTrue('Maintenance page' in content)

        # With tween enabled, allow to by-pass maintenance
        settings = {
            'mako.directories': 'pyramid_maintenance:tests/templates',
            'pyramid.tweens': 'pyramid_maintenance.tween_maintenance',
            'pyramid_maintenance.permissions': 'view',
            'pyramid_maintenance.template': 'maintenance.mako',
        }
        app = self._get_app(settings)
        content = app.get('/something')
        self.assertTrue('This is something' in content)

        # With tween enabled, allow to by-pass maintenance, several permissions in different forms
        settings = {
            'mako.directories': 'pyramid_maintenance:tests/templates',
            'pyramid.tweens': 'pyramid_maintenance.tween_maintenance',
            'pyramid_maintenance.permissions': 'admin\r\n view , ',
            'pyramid_maintenance.template': 'maintenance.mako',
        }
        app = self._get_app(settings)
        content = app.get('/something')
        self.assertTrue('This is something' in content)



from pyramid.interfaces import IRootFactory
from pyramid.response import Response
from pyramid.renderers import render

import re


def tween_maintenance(handler, registry):
    """
        Tween maintenance

        To enable it, please add in your .ini file:
        pyramid.tweens = pyramid_maintenance.tween_maintenance

        # List of permissions (separeted by comma, space, carriage return and/or new line).
        # The pyramid app isn't in maintenance mode for people who have one of these permissions.
        pyramid_maintenance.permissions = permission1, permission2
        # Relative path from defined template location directories.
        pyramid_maintenance.template = template.mako
        
    """
    permissions = filter(bool, map(str.strip, re.split(',|\t|\s', registry.settings['pyramid_maintenance.permissions'])))
    template = registry.settings['pyramid_maintenance.template']

    def doer(request):
        rootfactory = request.registry.queryUtility(IRootFactory)

        for permission in permissions:
            if request.has_permission(permission, rootfactory):
                return handler(request)

        return Response(render(template, {'title':''}, request))


    return doer



#! /usr/bin/env python
# -*- coding: utf-8 -*-

############################################################################
#
# Copyright © 2014 Benoît Pineau <beny@sickless.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the GNU Public License,
# Version 2.0 (GPL). A copy of the GPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
############################################################################

import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()
with open(os.path.join(here, 'CHANGES.txt')) as f:
    CHANGES = f.read()

requires = [
    'pyramid',
    ]

setup(name='pyramid_maintenance',
      version='0.2',
      description='pyramid_maintenance',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        ],
      author='Benoît Pineau',
      author_email='beny@sickless.net',
      url='',
      keywords='web pyramid pylons',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires + ['webtest', 'pyramid_mako'],
      test_suite="pyramid_maintenance",
      entry_points="""\
      [paste.app_factory]
      main = pyramid_maintenance:main
      """,
      )
